SystemOrganization addCategory: #Everett!
SystemOrganization addCategory: #'Everett-Tests'!

TestCase subclass: #EntityManagerTestCase
	instanceVariableNames: 'em'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

!EntityManagerTestCase methodsFor: 'running' stamp: 'EricShahkarami 7/2/2013 11:47'!
setUp 

	em := EntityManager new.! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 18:50'!
testEntityMatches

	| e1 e2 pc1 bc1 bc2 c1 c2 c3 s1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: bc1 toEntity: e1.
	em addComponent: bc2 toEntity: e2.
	
	c1 := Set newFrom: #(#position).
	c2 := Set newFrom: #(#velocity).
	c3 := Set newFrom: #(#position #velocity).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesContainingComponents: c1) = s1 ).
	self assert: ( (em entitiesContainingComponents: c2) = s2 ).
	self assert: ( (em entitiesContainingComponents: c3) = s1 )! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 23:57'!
testRemoveComponent

	| e pc vc c1 c2 s |
	
	e := em createEntity.

	pc := PositionComponent position: 3@7.
	vc := VelocityComponent velocity: 1@1.
	
	em addComponent: pc toEntity: e.
	em addComponent: vc toEntity: e.
	em removeComponent: (PositionComponent identity) fromEntity: e.
		
	c1 := Set newFrom: #(#velocity).
	c2:= Set newFrom: #(#position velocity).
	
	s := Set new.
	s add: e.
	
	self assert: ( (em entitiesContainingComponents: c1) = s ).
	self assert: ( (em entitiesContainingComponents: c2) isEmpty = true ).! !

TestCase subclass: #SystemManagerTestCase
	instanceVariableNames: 'em sm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:51'!
setUp

	em := EntityManager new.
	sm := SystemManager new! !

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/5/2013 15:55'!
testSystemUpdate

	"Tests that e1 and e2 have their positions updated but not e3, which doesn't have a velocity component"

	| e1 e2 e3 pc1 pc2 pc3 vc1 vc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.

	pc1 := PositionComponent position: 3@4.
	pc2 := PositionComponent position: 1@1.
	pc3 := PositionComponent position: 9@9.
	vc1 := VelocityComponent velocity: 1@1.
	vc2 := VelocityComponent velocity: 2@0.

	em addComponent: pc1 toEntity: e1.
	em addComponent: pc2 toEntity: e2.
	em addComponent: pc3 toEntity: e3.
	em addComponent: vc1 toEntity: e1.
	em addComponent: vc2 toEntity: e2.
	
	sm addSystem: (MovementSystem new).
	sm updateEntitiesOf: em afterDeltaTime: 1.
	
	self assert: ( (pc1 position) = (4@5) ).
	self assert: ( (pc2 position) = (3@1) ).
	self assert: ( (pc3 position) = (9@9) )! !

Object subclass: #EntityFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityFactory commentStamp: '<historical>' prior: 0!
I am a factory that creates entities with some specified component instances. Subclass me to specify which components.!

!EntityFactory methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/7/2013 00:18'!
createEntity: anEntityManager 

	| entity components |
	
	entity := anEntityManager createEntity.
	components := self obtainComponents.
	components do: [ :component | anEntityManager addComponent: component toEntity: entity ].
	^entity! !

!EntityFactory methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/7/2013 00:20'!
obtainComponents
	
	^(Set new)! !

Object subclass: #EntityManager
	instanceVariableNames: 'entityTable nextAvailableEntity changedEntities inactiveEntities removedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityManager commentStamp: '<historical>' prior: 0!
I am responsible for creating, removing and tracking all entities and their components.
!

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/7/2013 13:57'!
activateEntity: anEntity 

	inactiveEntities remove: anEntity.
	self changeEntity: anEntity! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 19:25'!
addComponent: aComponentInstance toEntity: anEntity

	(anEntity < nextAvailableEntity and: anEntity >= 0) ifTrue: 
	[  
		self entityAt: anEntity putComponent: aComponentInstance.
		self changeEntity: anEntity
	]! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 23:54'!
changeEntity: anEntity

	((changedEntities occurrencesOf: anEntity) > 1) ifFalse: [ changedEntities add: anEntity ]! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:34'!
changedEntitiesDo: aBlock

	(changedEntities asSet) do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 19:39'!
clearChangedEntities

	self changedEntitiesDo: [ :entity | changedEntities remove: entity ]! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 19:39'!
clearRemovedEntities

	self removedEntitiesDo: [ :entity | removedEntities remove: entity ]! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 09:38'!
component: aComponentIdentifier fromEntity: anEntity

	| componentsOfEntity |
	
	componentsOfEntity := self entityAt: anEntity.
	^(componentsOfEntity at: aComponentIdentifier)! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 19:27'!
createEntity

	entityTable at: nextAvailableEntity put: (Dictionary new).	
	self changeEntity: nextAvailableEntity.
	nextAvailableEntity := nextAvailableEntity + 1.
	^(nextAvailableEntity - 1)
! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/7/2013 13:49'!
deactivateEntity: anEntity

	inactiveEntities add: anEntity! !

!EntityManager methodsFor: 'testing' stamp: 'EricShahkarami 7/5/2013 08:34'!
doesEntity: anEntity containComponents: aSetOfComponents

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^( (aSetOfComponents intersection: entityComponents) = aSetOfComponents )! !

!EntityManager methodsFor: 'accessing' stamp: 'eshkrm 7/6/2013 18:50'!
entitiesContainingComponents: aSetOfComponents

	^( self entitiesSelect: [ :entity | self doesEntity: entity containComponents: aSetOfComponents ] )! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 23:51'!
entitiesDo: aBlock

	entityTable keysDo: [ :entity | aBlock value: entity ].! !

!EntityManager methodsFor: 'enumerate' stamp: 'EricShahkarami 7/2/2013 11:22'!
entitiesSelect: aBlock

	| selectEntities |
	
	selectEntities := Set new.
	entityTable keysDo: [ :entity | (aBlock value: entity) ifTrue: [ selectEntities add: entity ]  ].
	^selectEntities! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity
	
	^(entityTable at: anEntity)! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity putComponent: aComponentInstance 
		
	(entityTable at: anEntity) at: (aComponentInstance class identity) put: aComponentInstance. ! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 13:53'!
inactiveEntitiesDo: aBlock 

	inactiveEntities do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/7/2013 13:47'!
initialize 
	
	super initialize.
	entityTable := Dictionary new.
	changedEntities := Bag new.
	inactiveEntities := Set new.
	removedEntities := Set new.
	nextAvailableEntity := 0! !

!EntityManager methodsFor: 'modify-entities' stamp: 'eshkrm 7/6/2013 19:25'!
removeComponent: aComponentIdentifier fromEntity: anEntity

	| components |
	
	components := self entityAt: anEntity.
	components removeKey: aComponentIdentifier.
	self changeEntity: anEntity
! !

!EntityManager methodsFor: 'modify-entities' stamp: 'EricShahkarami 7/1/2013 11:17'!
removeEntity: anEntity	
	"Removes entity from entityTable and places entity in removed pile."	
	
	entityTable removeKey: anEntity.
	removedEntities add: anEntity
	! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:15'!
removedEntitiesDo: aBlock 

	removedEntities do: [ :entity | aBlock value: entity ]! !

Object subclass: #EverettComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettComponent commentStamp: '<historical>' prior: 0!
I hold data for entities. Systems modify me.!

!EverettComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 13:52'!
identity
	^self subclassResponsibility! !

EverettComponent subclass: #PositionComponent
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PositionComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the position of an entity.!

!PositionComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:09'!
identity
	^#position! !

!PositionComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
position: aPosition

	^(self new position: aPosition)! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
position

	^position! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:02'!
position: aPosition 
	
	position := aPosition! !

EverettComponent subclass: #VelocityComponent
	instanceVariableNames: 'velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!VelocityComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the velocity of an entity.!

!VelocityComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:27'!
identity 	
	^#velocity! !

!VelocityComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
velocity: aVelocity

	^(self new velocity: aVelocity)! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity

	^velocity ! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity: aVelocity
	
	velocity := aVelocity! !

Object subclass: #EverettSystem
	instanceVariableNames: 'updateInterval lastProcessed interestedComponents interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettSystem commentStamp: '<historical>' prior: 0!
I am system that processes entities.
!

!EverettSystem class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 14:20'!
identity
	^self subclassResponsibility ! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/6/2013 19:12'!
initialize

	updateInterval := 0.
	lastProcessed := 0.
	interestedComponents := Set new.
	interestedEntities := Set new! !

!EverettSystem methodsFor: 'accessing' stamp: 'EricShahkarami 7/5/2013 11:28'!
interestedEntities

	^interestedEntities! !

!EverettSystem methodsFor: 'private' stamp: 'eshkrm 7/6/2013 18:58'!
resolveChangedEntitiesFrom: anEntityManager
	
	anEntityManager changedEntitiesDo: [ :entity | self verifyEntity: entity from: anEntityManager ]! !

!EverettSystem methodsFor: 'private' stamp: 'eshkrm 7/7/2013 13:57'!
resolveInactiveEntitiesFrom: anEntityManager

	anEntityManager inactiveEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!EverettSystem methodsFor: 'private' stamp: 'eshkrm 7/6/2013 19:18'!
resolveRemovedEntitiesFrom: anEntityManager 

	anEntityManager removedEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!EverettSystem methodsFor: 'processing' stamp: 'eshkrm 7/7/2013 14:17'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
								
	self resolveChangedEntitiesFrom: anEntityManager.
	self resolveInactiveEntitiesFrom: anEntityManager.	
	self resolveRemovedEntitiesFrom: anEntityManager.
		
	lastProcessed := lastProcessed + dt.
	(lastProcessed >= updateInterval) ifTrue: 
	[ 
		interestedEntities do: [ :entity | self updateEntity: entity from: anEntityManager afterDeltaTime: dt ].
		lastProcessed := 0
	]! !

!EverettSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	^self subclassResponsibility! !

!EverettSystem methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 14:46'!
updateInterval: anUpdateInterval
	
	updateInterval := anUpdateInterval ! !

!EverettSystem methodsFor: 'testing' stamp: 'EricShahkarami 7/5/2013 11:32'!
verifyEntity: anEntity from: anEntityManager

	(anEntityManager doesEntity: anEntity containComponents: interestedComponents) 
	ifTrue: 
	[  
		(interestedEntities includes: anEntity) ifFalse: [ interestedEntities add: anEntity ].
	]
	ifFalse:
	[ 
		(interestedEntities includes: anEntity) ifTrue: [ interestedEntities remove: anEntity ]
	]! !

EverettSystem subclass: #MovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!MovementSystem commentStamp: '<historical>' prior: 0!
I am a system that updates the position of entities.!

!MovementSystem class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:55'!
identity 

	^#movement! !

!MovementSystem methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/5/2013 11:19'!
initialize
	
	super initialize.
	interestedComponents add: (PositionComponent identity); add: (VelocityComponent identity).! !

!MovementSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 15:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| pc vc |
	
	pc := anEntityManager component: (PositionComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	pc position: ( pc position + (dt * (vc velocity)) )! !

Object subclass: #SystemManager
	instanceVariableNames: 'systems priorities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!SystemManager commentStamp: '<historical>' prior: 0!
I manage all running and inactive systems.!

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/1/2013 15:18'!
addSystem: aSystemInstance 
	
	self addSystem: aSystemInstance withPriority: 0 ! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/2/2013 11:29'!
addSystem: aSystemInstance withPriority: aPriority

	systems at: (aSystemInstance class identity) put: aSystemInstance.
	priorities add: ( (aSystemInstance class identity)->aPriority )! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/1/2013 15:53'!
assignSystem: aSystemIdentifier aPriorityOf: aPriority

	priorities add: (aSystemIdentifier->aPriority)! !

!SystemManager methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/1/2013 15:51'!
initialize
	
	super initialize.
	systems := Dictionary new.
	priorities := SortedCollection sortBlock: [ :p1 :p2 | p1 value <= p2 value ]! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/2/2013 11:30'!
removeSystem: aSystemIdentifier

	| systemPriority |

	systems removeKey: aSystemIdentifier.
	systemPriority := priorities select: [ :assoc | assoc key = aSystemIdentifier ].
	priorities remove: (systemPriority at: 1)! !

!SystemManager methodsFor: 'processing' stamp: 'eshkrm 7/6/2013 19:40'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	"priorities do: [ :assoc | (systems at: (assoc key)) updateEntitiesOf: anEntityManager afterDeltaTime: dt ]."
	systems do: [ :system | system updateEntitiesOf: anEntityManager afterDeltaTime: dt ].
		
	anEntityManager clearChangedEntities.
	anEntityManager clearRemovedEntities
	! !
